// This file is made available under Elastic License 2.0.
// This file is based on code available under the Apache license here:
//   https://github.com/apache/incubator-doris/blob/master/be/src/olap/task/engine_checksum_task.h

// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

#ifndef STARROCKS_BE_SRC_OLAP_TASK_ENGINE_CHECKSUM_TASK_H
#define STARROCKS_BE_SRC_OLAP_TASK_ENGINE_CHECKSUM_TASK_H

#include "gen_cpp/AgentService_types.h"
#include "storage/olap_define.h"
#include "storage/task/engine_task.h"

namespace starrocks {

// base class for storage engine
// add "Engine" as task prefix to prevent duplicate name with agent task
class EngineChecksumTask : public EngineTask {
public:
    OLAPStatus execute() override;

    EngineChecksumTask(MemTracker* mem_tracker, TTabletId tablet_id, TSchemaHash schema_hash, TVersion version,
                       TVersionHash version_hash, uint32_t* checksum);

    ~EngineChecksumTask() override = default;

private:
    OLAPStatus _compute_checksum();

    std::unique_ptr<MemTracker> _mem_tracker;

    TTabletId _tablet_id;
    TSchemaHash _schema_hash;
    TVersion _version;
    TVersionHash _version_hash;
    uint32_t* _checksum;
}; // EngineTask

} // namespace starrocks
#endif //STARROCKS_BE_SRC_OLAP_TASK_ENGINE_CHECKSUM_TASK_H
